/* Blinker
   Copyright 2011-2019 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef BLINKER_PONG_H
#define BLINKER_PONG_H

#include <string>
#include <vector>

#include <BlinkenLib/BlinkenFrame.h>

#include "Color.h"
#include "ColorFile.h"
#include "File.h"
#include "Format.h"
#include "FormatFile.h"
#include "Game.h"
#include "Mgrs.h"
#include "Module.h"
#include "OutStreamFile.h"
#include "Time.h"
#include "TimeCallee.h"

namespace Blinker {

/// pong game
class Pong: public Game, public TimeCallee
{
public:
  /**
   * @brief constructor
   * @param[in] name module name
   * @param[in] mgrs managers
   * @param[in] dirBase base directory
   */
  Pong(const std::string &name, Mgrs &mgrs, const Directory &dirBase);

  /// virtual destructor
  virtual ~Pong();

private:
  /// copy constructor disabled
  Pong(const Pong &that);

  /// assignment operator disabled
  const Pong & operator=(const Pong &that);

public:
  /// check for update of configuration (derived game), return true on update
  virtual bool updateConfigGame();

protected:
  /// re-initialize game (e.g. due to config change)
  virtual void reinitialize();

  /// redraw current game image, expected to call sendFrame() at end
  virtual void redraw();

  /// callback when requested time reached
  virtual void timeCall();

  /// move ball out of the field and halt it
  void hideBall();

  /// start ball
  void startBall();

protected:
  ColorFile m_fileBallColor; ///< color file for ball color
  ColorFile m_fileLineColor; ///< color file for center line
  ColorData m_ballColor;     ///< ball color
  ColorData m_lineColor;     ///< center line color
  int       m_ballPosX;      ///< ball position X
  int       m_ballPosY;      ///< ball position Y
  int       m_ballDirX;      ///< ball direction X
  int       m_ballDirY;      ///< ball direction Y
}; // class Canvas

} // namespace Blinker

#endif // #ifndef BLINKER_PONG_H

